require_relative 'player'
require_relative 'game_turn'
require_relative 'treasure_trove'
require 'csv'

module StudioGame
  class Game
  
    attr_reader :title
  
    def initialize(title)
      @title  = title
      @players = []
    end
  
    def add_player(player_name)
      @players << player_name
    end
  
    def load_players(file_name)
      #File.readlines(file_name).each do |line|
        #add_player(Player.from_csv(line))
        CSV.foreach(file_name) do |row|
          player = Player.new(row[0], row[1].to_i)
          add_player(player)
        end
      #end
    end
  
    def save_high_scores(file_name="high_scores.txt")
      File.open(file_name, "w") do |file|
        file.puts "#{title} High Scores:"
        puts "\n#{@title} High Scores:"
        @players.sort.each do |player|
          file.puts write_name_score(player)
        end
      end
    end
  
    def write_name_score(player)
      "#{player.name.ljust(20,'.')}#{player.score}"
    end
  
    def print_name_and_health(player)
      "#{player.name} (#{player.health})"
    end
  
    def print_stats
      strong_player, wimpy_player = @players.partition { |p| p.strong? }
    
      puts "\n#{@title} Statistics\n"
    
      puts "\n#{strong_player.size} strong players:"
      strong_player.each do |player|
        print_name_and_health(player)
      end
    
      puts "\n#{wimpy_player.size} wimpy players:"
      wimpy_player.each do |player|
        print_name_and_health(player)
      end
    
      @players.each do |player|
        puts "\n#{player.name}'s point totals:"
        player.each_found_treasure {|treasure| puts "#{treasure.points} total #{treasure.name} points"}
        puts "#{player.points} grand total points"
      end
    
      puts "\n#{@title} High Scores:"
      @players.sort.each do |player|
        puts write_name_score(player)
      end
    
      puts "\nTotal points found during the game: #{total_points}"
    end
  
    def play(rounds)
      puts "There are #{@players.size} players in #{@title}:"
      @players.each do |player|
        puts player
      end
    
      treasures = TreasureTrove::TREASURES
      puts "\nThere are #{treasures.size} treasures available!"
      treasures.each do |treasure|
        name = treasure.name.to_s
        puts "#{name.ljust(20,'.')}#{treasure.points}pts"
      end
    
      1.upto(rounds) do |round_number|
        if block_given?
          break if yield
        end
        puts "\nRound number: #{round_number}"
        @players.each do |player|
          GameTurn.take_turn(player)
          puts player
        end
      end
    end
  
    def total_points
      @players.reduce(0) { |sum, player| sum + player.points}
    end
  
  end

  if __FILE__ == $0
    player1 = Player.new("moe")
    player2 = Player.new("larry",60)
    player3 = Player.new("curly",125)

    knuckleheads = Game.new("Knuckleheads")
    knuckleheads.add_player(player1)
    knuckleheads.add_player(player2)
    knuckleheads.add_player(player3)
    knuckleheads.play(3)
    knuckleheads.print_stats
  end
end